package kz.project.printedFormsService.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import kz.project.printedFormsService.config.SecurityContextUtils;
import kz.project.printedFormsService.data.dto.DocumentByDateDto;
import kz.project.printedFormsService.data.dto.DocumentByTemplateDto;
import kz.project.printedFormsService.exception.ValidationException;
import kz.project.printedFormsService.logging.ProcessLogger;
import kz.project.printedFormsService.service.DocumentsService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.apache.bcel.generic.LocalVariableGen;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;

@RestController
@Slf4j
@Tag(name = "Documents Controller", description = "API DocumentsService")
public class DocumentsController {
    private static final Logger LOGGER = LoggerFactory.getLogger(DocumentsController.class);
    private final DocumentsService documentsService;

    public static final String BASE_PATH = "/api/documents";
    public static final String BY_DAY = BASE_PATH + "/by-day";
    public static final String BY_TEMPLATE = BASE_PATH + "/by-template";
    private ProcessLogger processLogger;

    public DocumentsController(DocumentsService documentsService,
                               ProcessLogger processLogger) {
        this.documentsService = documentsService;
        this.processLogger = processLogger;
    }

    @GetMapping(BY_DAY)
    @Operation(description = "Метод для получения данных по общему количеству генерации по всем доступным пользователю шаблонам")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = DocumentByDateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка валидации параметров запроса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity getDocumentsCountByDay(
            @RequestParam("startDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate startDate,
            @RequestParam("endDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate endDate) {
        try {
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение данных генераций всех отчётов за диапазон времени");
            Map<String, List<String>> projectRoleMap = SecurityContextUtils.getProjectRoleMap();
            DocumentByDateDto documentsByDate = documentsService.getDocumentCountsByDate(startDate, endDate, projectRoleMap.keySet());
            processLogger.finish(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение данных генераций всех отчётов за диапазон времени");
            return ResponseEntity.ok(documentsByDate);
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение данных генераций всех отчётов за диапазон времени");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

    @GetMapping(BY_TEMPLATE)
    @Operation(description = "Метод для получения данных по общему количеству генерации по каждому доступному пользователю шаблонам")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = DocumentByTemplateDto.class))
                    }),
            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка валидации параметров запроса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity getDocumentsCountByTemplate(
            @RequestParam("startDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate startDate,
            @RequestParam("endDate")
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE) LocalDate endDate) {
        try {
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение данных генераций по шаблону за диапазон времени");
            Map<String, List<String>> projectRoleMap = SecurityContextUtils.getProjectRoleMap();
            DocumentByTemplateDto documentCountsByTemplate = documentsService.getDocumentCountsByTemplate(startDate, endDate, projectRoleMap.keySet());
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение данных генераций по шаблону за диапазон времени");
            return ResponseEntity.ok(documentCountsByTemplate);
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение данных генераций по шаблону за диапазон времени");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }
}
