package kz.project.printedFormsService.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import kz.project.printedFormsService.data.dto.TemplateDto;
import kz.project.printedFormsService.data.dto.TemplateResponseDto;
import kz.project.printedFormsService.data.dto.TemplateShortDto;
import kz.project.printedFormsService.exception.ValidationException;
import kz.project.printedFormsService.logging.ProcessLogger;
import kz.project.printedFormsService.service.TemplateService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(TemplateController.BASE_PATH)
@Slf4j
@Tag(name = "Template Controller", description = "API TemplateService")
public class TemplateController {
    private static final Logger LOGGER = LoggerFactory.getLogger(TemplateController.class);
    public static final String BASE_PATH = "/api/template";
    public static final String GET_TEMPLATE = "/{id}";
    public static final String GET_TEMPLATE_DATA = "get/{id}";
    public static final String SAVE = "/save";
    public static final String EDIT = "/edit";
    public static final String DELETE = "/delete/{code}";
    public static final String GET_ALL = "/all";
    public static final String GET_ALL_BY_CODE = "/allVersion/{code}";

    private final TemplateService service;
    private final ProcessLogger processLogger;

    public TemplateController(TemplateService service, ProcessLogger processLogger) {
        this.service = service;
        this.processLogger = processLogger;
    }

    @Hidden
    @GetMapping(GET_TEMPLATE)
    @Operation(description = "Метод для получения шаблона по идентификатору")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = Map.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity getTemplate(@Parameter(name = "id", required = true) @PathVariable Long id) throws ValidationException, JsonProcessingException {
        try {
            processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Получение шаблона по идентификатору");
            Map<String, byte[]> templateMap = service.getTemplate(id);
            return ResponseEntity.ok(templateMap);
        } catch (Exception e){
            processLogger.error(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Получение шаблона по идентификатору");
            LOGGER.error(e.getMessage(), e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

    @GetMapping(GET_TEMPLATE_DATA)
    @Operation(description = "Метод для получения шаблона по идентификатору")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = TemplateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity getTemplateData(@Parameter(name = "id", required = true) @PathVariable Long id) throws ValidationException {
        try {
            processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Получение данных шаблона по идентификатору");
            TemplateDto templateData = service.getTemplateData(id);
            processLogger.finish(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Получение данных шаблона по идентификатору");
            return ResponseEntity.ok(templateData);
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.finish(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Получение данных шаблона по идентификатору");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

    @PutMapping(SAVE)
    @Operation(description = "Метод сохранения шаблона")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = TemplateDto.class))
                    }),
            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })
    public ResponseEntity saveTemplate(@Parameter(name = "data") @RequestParam(name = "data", required = false) MultipartFile data,
                                                    @Parameter(name = "header") @RequestParam(name = "header", required = false) MultipartFile header,
                                                    @RequestParam TemplateDto dto


    ) throws IOException, ValidationException {
        try {
            if (data == null) {
                throw new ValidationException("Не переданы обязательные поля: data", 13);
            }
            if (header != null) {
                return ResponseEntity.ok(service.save(dto, List.of(data, header)));
            } else {
                return ResponseEntity.ok(service.save(dto, List.of(data)));
            }
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Сохранение шаблона");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }


    @PutMapping(EDIT)
    @Operation(description = "Метод для сохранение изменения")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = TemplateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    public ResponseEntity editTemplate(@Parameter(name = "data") @RequestParam(name = "data", required = false) MultipartFile data,
                                                            @Parameter(name = "header") @RequestParam(name = "header", required = false) MultipartFile header,
                                                            @RequestParam TemplateDto dto) throws IOException, ValidationException {
        try {
            //        if (data == null) {
//            throw new ValidationException("Не переданы обязательные поля: data", 13);
//        }
            if (header != null) {
                return ResponseEntity.ok(service.edit(dto, List.of(data, header)));
            } else if (data != null) {
                return ResponseEntity.ok(service.edit(dto, List.of(data)));
            } else {
                return ResponseEntity.ok(service.edit(dto, null));
            }
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Редактирование шаблона");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

    @DeleteMapping(DELETE)
    @Operation(description = "Метод для удаления шаблона по идентификатору")
    public ResponseEntity deleteTemplate(@Parameter(name = "code", required = true) @PathVariable("code") String code) throws ValidationException {
        try {
            processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Удаление шаблона");
            service.delete(code);
            return ResponseEntity.ok("delete is success");
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Удаление шаблона");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

    @GetMapping(GET_ALL)
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = Page.class))
                    }),
            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })

    @Operation(description = "Метод для получения всех шаблонов")
    public ResponseEntity getAll(@Parameter(name = "isActive") @RequestParam(value = "isActive", required = false) Boolean isActive,
                                         @Parameter(name = "page") @RequestParam(value = "page", defaultValue = "0") Integer page,
                                         @Parameter(name = "size") @RequestParam(value = "size", defaultValue = "50") Integer size) {

        try {
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение списка шаблонов");
            Page<TemplateShortDto> allTemplates = service.getAllTemplate(isActive, PageRequest.of(page, size));
            processLogger.finish(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение списка шаблонов");
            return ResponseEntity.ok(allTemplates);
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.error(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение списка шаблонов");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

    @GetMapping(GET_ALL_BY_CODE)
    @Operation(description = "Метод для получения всех шаблонов")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = Page.class))
                    }),
            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })
    public ResponseEntity getAllByCode(@Parameter(name = "code") @PathVariable(value = "code") String code,
                                          @Parameter(name = "page") @RequestParam(value = "page", defaultValue = "0") Integer page,
                                          @Parameter(name = "size") @RequestParam(value = "size", defaultValue = "50") Integer size) throws ValidationException {
        try {
            Page<TemplateDto> pageDTO = service.getAllTemplateByCode(code, PageRequest.of(page, size));
            return ResponseEntity.ok(pageDTO);
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение всех шаблонов по коду");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }

}
