package kz.project.printedFormsService.logging;

import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.http.HttpServletRequest;
import kz.project.printedFormsService.converter.JwtAuthConverter;
import org.apache.tomcat.util.http.fileupload.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.format.datetime.DateFormatter;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.nio.file.attribute.PosixFilePermission;
import java.text.ParseException;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;
import java.util.stream.Collectors;

/* Класс для логгирования старта и завершения процессов в формате SIEM и на естественном языке */
@Component
public class ProcessLogger {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProcessLogger.class);
    private static final String DEFAULT_FILENAME = "siem-processes.log";
    private static final int DEFAULT_FILE_SIZE = 100;
    private static final String DEFAULT_DELIMITER = " ";
    private static final String DEFAULT_START_PREFIX = "Process start: ";
    private static final String DEFAULT_FINISH_PREFIX = "Process finish: ";
    private static final String DEFAULT_ERROR_PREFIX = "Process error: ";

    @Value("${process-logger.file-name}")
    private String logFileName;
    @Value("${process-logger.file-max-size-mb}")
    private int logFileSize;
    @Value("${process-logger.delimiter}")
    private String delimiter;
    @Value("${spring.application.name}")
    private String serviceName;
    @Value("${process-logger.start-prefix}")
    private String startPrefix;
    @Value("${process-logger.finish-prefix}")
    private String finishPrefix;
    @Value("${process-logger.error-prefix}")
    private String errorPrefix;

    private DateFormatter dateFormatter;
    private Locale locale;
    private ZoneId zoneId;

    public ProcessLogger(DateFormatter dateFormatter, Locale locale, ZoneId zoneId) {
        this.dateFormatter = dateFormatter;
        this.locale = locale;
        this.zoneId = zoneId;
    }

    public void start(Class javaClass, String processName, String description){
        write(javaClass, processName, startPrefix != null ? startPrefix : DEFAULT_START_PREFIX, description);
    }

    public void finish(Class javaClass, String processName, String description){
        write(javaClass, processName, finishPrefix != null ? finishPrefix : DEFAULT_FINISH_PREFIX, description);
    }

    public void error(Class javaClass, String processName, String description){
        write(javaClass, processName, errorPrefix != null ? errorPrefix : DEFAULT_ERROR_PREFIX, description);
    }

    public void write(Class javaClass, String processName, String descriptionPrefix, String description){
        String fileName = this.logFileName != null ? this.logFileName : DEFAULT_FILENAME;

        try {
            HttpServletRequest request = getRequest();

            String logRow = getLogRow(request, javaClass.getSimpleName(), processName, descriptionPrefix, description, fileName);

            File file = new File(fileName);
            controlFileSize(file);
            boolean isCreated = file.createNewFile();
            if (!isValidFile(file)) return;
            if (isUnixSystem()){
                setFilePermissions(file);
            }

            FileWriter writer = new FileWriter(fileName, true);
            writer.append(logRow).append(System.lineSeparator());
            writer.close();
        } catch (FileNotFoundException fnfe){
            LOGGER.error("Ошибка чтения/записи файла {}. Недостаточно прав.", fileName);
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
        }
    }

    private String getLogRow(HttpServletRequest request, String className, String processName, String descriptionPrefix, String description, String fileName){
        String delimiter = this.delimiter != null ? this.delimiter : DEFAULT_DELIMITER;

        try {
            String ip = request.getRemoteAddr();
            String userName = getUserName(request);

            return String.format(
                    "%s%s%s.%s%s%s%s%s%s%s%s%s%s",
                    serviceName,
                    delimiter,
                    className,
                    processName,
                    delimiter,
                    getTodayDateTimeString(),
                    delimiter,
                    userName != null ? userName : "system-request",
                    delimiter,
                    ip,
                    delimiter,
                    descriptionPrefix,
                    description
            );
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            return "";
        }
    }

    private String getUserName(HttpServletRequest request) throws ParseException {
        String authHeader = request.getHeader(HttpHeaders.AUTHORIZATION);
        String userName = null;

        if (authHeader != null){
            userName = JwtAuthConverter.getUsername(authHeader.substring(7));
        } else {
            authHeader = request.getHeader("X-API-Key");
            if (authHeader != null) userName = authHeader.substring(9);
        }

        return userName;
    }

    private boolean isValidFile(File file){
        boolean isValid = file.isFile();
        if (!isValid) LOGGER.warn("{} не является файлом. Проверьте конфигурацию.", file.getName());
        return isValid;
    }

    private boolean isOverwhelmed(File file){
        int maxSizeMegabytes = logFileSize > 0 ? logFileSize : DEFAULT_FILE_SIZE;
        long fileSizeMegabytes = file.length() / (1024*1024);
        return fileSizeMegabytes >= maxSizeMegabytes;
    }

    private boolean isUnixSystem(){
        String osName = System.getProperty("os.name").toLowerCase();
        return osName.contains("nix") || osName.contains("nux");
    }

    private void copyFile(File file) throws IOException {
        String filePath = this.logFileName != null ? this.logFileName : DEFAULT_FILENAME;
        String todayString = getTodayDateTimeString();
        String copyFilePath = String.format("%s%s", filePath, todayString.replace(" ", "").replace(".", "").replace(":", ""));

        Path copyPath = Paths.get(copyFilePath);
        Path originalPath = file.toPath();

        Files.copy(originalPath, copyPath, StandardCopyOption.REPLACE_EXISTING);
        LOGGER.info("Файл лога {} превысил допустимый размер, и был скопирован в {}", filePath, copyFilePath);
    }

    private void controlFileSize(File file) throws IOException {
        if (isOverwhelmed(file)){
            copyFile(file);
            file.delete();
        }
    }

    private void setFilePermissions(File file) throws IOException {
        Set<PosixFilePermission> permissions = Arrays.stream(PosixFilePermission.values()).collect(Collectors.toSet());
        Files.setPosixFilePermissions(file.toPath(), permissions);
    }

    private String getTodayDateTimeString(){
        Instant instant = new Date().toInstant();
        ZonedDateTime zonedDateTime = instant.atZone(this.zoneId);

        Date today = Date.from(zonedDateTime.toInstant());
        return dateFormatter.print(today, locale);
    }

    private HttpServletRequest getRequest(){
        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    }
}
