package kz.project.printedFormsService.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import kz.project.printedFormsService.controller.TemplateController;
import kz.project.printedFormsService.data.dto.TemplateDataForReportDto;
import kz.project.printedFormsService.data.dto.TemplateDto;
import kz.project.printedFormsService.data.dto.TemplateResponseDto;
import kz.project.printedFormsService.data.dto.TemplateShortDto;
import kz.project.printedFormsService.data.entity.TemplateEntity;
import kz.project.printedFormsService.data.entity.TemplateEntityVersion;
import kz.project.printedFormsService.data.entity.TemplateFileInfoEntity;
import kz.project.printedFormsService.data.repository.*;
import kz.project.printedFormsService.exception.ValidationException;
import kz.project.printedFormsService.logging.ProcessLogger;
import kz.project.printedFormsService.service.TemplateService;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class TemplateServiceImpl implements TemplateService {
    private final TemplateRepository repository;
    private final TemplateVersionRepository versionRepository;
    private final DTemplateTypeRepository dTemplateTypeRepository;

    private final TemplateFileInfoRepository fileInfoRepository;
    private final DocumentRepository documentRepository;
    private ProcessLogger processLogger;

    public TemplateServiceImpl(TemplateRepository repository,
                               TemplateVersionRepository versionRepository,
                               DTemplateTypeRepository dTemplateTypeRepository,
                               TemplateFileInfoRepository fileInfoRepository,
                               DocumentRepository documentRepository,
                               ProcessLogger processLogger) {
        this.repository = repository;
        this.versionRepository = versionRepository;
        this.dTemplateTypeRepository = dTemplateTypeRepository;
        this.fileInfoRepository = fileInfoRepository;
        this.documentRepository = documentRepository;
        this.processLogger = processLogger;
    }

    @Override
    public Map<String, byte[]> getTemplate(Long id) throws ValidationException, JsonProcessingException {
        Map<String, byte[]> params = new HashMap<>();
        TemplateEntity templateEntity = repository.findById(id).orElse(null);
        if (templateEntity == null) throw new ValidationException("по данному параметру отсутствуют данные: " + id, 13);
        TemplateDataForReportDto templ = new TemplateDataForReportDto(templateEntity.getType().getCode(), templateEntity.getTemplate().getData(), templateEntity.getTemplateHeader() != null ? templateEntity.getTemplateHeader().getData() : null);
        params.put("type", templ.getType().getBytes(StandardCharsets.UTF_8));
        params.put("body", templ.getBody());
        params.put("header", templ.getHeader());

        processLogger.finish(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Получение шаблона по идентификатору");
        return params;

    }

    @Override
    @Transactional
    public TemplateDto save(TemplateDto dto, List<MultipartFile> files) throws IOException, ValidationException {
        processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Сохранение шаблона");

        String fieldName = validateData(dto);
        if (fieldName != null) {
            throw new ValidationException("Не переданы обязательные поля: " + fieldName, 13);
        }
        TemplateEntity template = repository.findByCode(dto.getCode()).orElse(null);
        if (template != null) {
            throw new ValidationException("Шаблон с таким кодом уже существует: " + dto.getCode(), 13);
        }
        TemplateEntity templateEntity = createTemplateEntity(dto, files);
        TemplateEntity save = repository.save(templateEntity);

        processLogger.finish(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Сохранение шаблона");

        return TemplateDto.toDto(save);

    }

    @Override
    @Transactional
    public TemplateResponseDto edit(TemplateDto dto, List<MultipartFile> files) throws IOException, ValidationException {
        processLogger.start(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Редактирование шаблона");

        String fieldName = validateData(dto);
        if (fieldName != null) {
            throw new ValidationException("Не переданы обязательные поля: " + fieldName, 13);
        }
        if (dto.getTemplateId() == null) {
            throw new ValidationException("Не переданы обязательные поля: templateId", 13);
        }
        if (repository.findById(dto.getTemplateId()).isEmpty())
            throw new ValidationException("Cо значением templateId=" + dto.getTemplateId() + "запись не существует", 13);

        TemplateEntity existingEntity = repository.findById(dto.getTemplateId()).get();
        TemplateEntity receivedEntity = createTemplateEntity(dto, files);

        if (!existingEntity.equals(receivedEntity)) {
            versionRepository.save(new TemplateEntityVersion(existingEntity));
        }

        processLogger.finish(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Редактирование шаблона");

        return TemplateResponseDto.toDto(repository.save(receivedEntity));

    }

    @Override
    public void delete(String code) throws ValidationException {
        TemplateEntity template = repository.findByCode(code).orElse(null);
        if (template == null) {
            throw new ValidationException("Не найден шаблон с таким кодом", 13);
        }
        repository.delete(template);
        processLogger.finish(this.getClass(), new Throwable().getStackTrace()[0].getMethodName(), "Удаление шаблона");
    }

    @Override
    public Page<TemplateShortDto> getAllTemplate(Boolean isActive, Pageable pageable) {
        if (isActive == null) return TemplateShortDto.toDtoList(repository.findAll(pageable));
        else if (isActive)
            return TemplateShortDto.toDtoList(repository.findAllByStatusTrue(pageable));
        else return TemplateShortDto.toDtoList(repository.findAllByStatusFalse(pageable));
    }

    @Override
    public TemplateDto getTemplateData(Long id) throws ValidationException {
        if (id == null) throw new ValidationException("Не переданы обязательные поля", 13);
        return TemplateDto.toDto(repository.findById(id).orElseThrow(() -> new ValidationException("Не найден шаблон с таким идентификатором", 13)));
    }

    @Override
    public Page<TemplateDto> getAllTemplateByCode(String code, Pageable pageable) throws ValidationException {
        processLogger.start(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение всех шаблонов по коду");

        if (code != null) {
            Page<TemplateEntity> allByCode = repository.findAllByCode(pageable, code);
            Page<TemplateEntityVersion> allVersionsByCode = versionRepository.findAllByCode(pageable, code);

            processLogger.finish(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение всех шаблонов по коду");

            if (allByCode != null && allVersionsByCode != null)
                return TemplateDto.toDtoList(allByCode, allVersionsByCode);

            if (allByCode != null)
                return TemplateDto.toDtoList(allByCode);
        }
        throw new ValidationException("Не переданы обязательные поля", 13);
    }


    private TemplateEntity createTemplateEntity(TemplateDto dto, List<MultipartFile> files) throws IOException, ValidationException {
        TemplateEntity templateEntity = new TemplateEntity();
        if (dto.getTemplateId() != null) {
            templateEntity.setId(dto.getTemplateId());
        }
        templateEntity.setCode(dto.getCode());
        if (files != null && !files.isEmpty() && files.get(0) != null) {
            if (dto.getTemplateFileId() != null) {
                templateEntity.setTemplate(updateTemplateFiles(files.get(0), dto.getTemplateFileId(), false));
            } else if (dto.getHeaderFileId() != null) {
                templateEntity.setTemplate(updateTemplateFiles(files.get(0), dto.getHeaderFileId(), false));
            } else {
                templateEntity.setTemplate(createTemplateFiles(files.get(0), false));
            }
        }
        templateEntity.setCode(dto.getCode());
        templateEntity.setName(dto.getName());
        templateEntity.setStatus(dto.getStatus());
        templateEntity.setProject(dto.getProject());
        templateEntity.setType(dTemplateTypeRepository.findByCode(dto.getType()).orElseThrow(() -> new ValidationException("Не найден справочное значение по коду: " + dto.getType(), 13)));
        templateEntity.setVersion(dto.getVersion() + 1);
        if (files != null && files.size() == 2 && files.get(1) != null) {
            if (dto.getHeaderFileId() != null) {
                templateEntity.setTemplateHeader(updateTemplateFiles(files.get(1), dto.getHeaderFileId(), true));
            } else {
                templateEntity.setTemplateHeader(createTemplateFiles(files.get(1), true));
            }
        }
        templateEntity.setStatus(dto.getStatus());
        templateEntity.setVersion(dto.getVersion() != null ? dto.getVersion() + 1 : 1);
        return templateEntity;
    }

    private TemplateFileInfoEntity createTemplateFiles(MultipartFile file, boolean isHeader) throws IOException {
        return fileInfoRepository.save(TemplateFileInfoEntity.builder()
                .isHeader(isHeader)
                .name(file.getOriginalFilename())
                .data(file.getResource().getContentAsByteArray())
                .hash(file.getResource().getContentAsByteArray().hashCode())
                .build());
    }

    private TemplateFileInfoEntity updateTemplateFiles(MultipartFile file, Long templateId, boolean isHeader) throws IOException {
        return fileInfoRepository.save(TemplateFileInfoEntity.builder()
                .isHeader(isHeader)
                .id(templateId)
                .name(file.getOriginalFilename())
                .data(file.getResource().getContentAsByteArray())
                .hash(Arrays.hashCode(file.getResource().getContentAsByteArray()))
                .build());
    }

    private String validateData(TemplateDto dto) {
        if (dto == null) {
            return "dto";
        } else if (dto.getCode() == null) {
            return "code";
        } else if (dto.getName() == null) {
            return "name";
        } else if (dto.getStatus() == null) {
            return "status";
        } else if (dto.getType() == null) {
            return "type";
        } else if (dto.getProject() == null) {
            return "project";
        }
        return null;
    }
}
